from django.test import TestCase
from django.contrib.auth.models import User, Group

from .models import Person, Place, Institution, Work, Event
from apis_core.apis_metainfo.models import Text, Collection, Source, Uri, UriCandidate
from apis_core.apis_vocabularies.models import PersonPlaceRelation
from apis_core.apis_labels.models import Label
from apis_core.apis_relations.models import PersonPlace, PlacePlace
from reversion.models import Version
from reversion import revisions as reversion
from apis_core.helper_functions.RDFparsers import GenericRDFParser

from datetime import datetime
from guardian.shortcuts import assign_perm, remove_perm, get_objects_for_user
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient
from django.contrib.auth.models import Permission
from django.db.models import Q


class RDFPersonParserTestCase(TestCase):
    #fixtures = ['fixtures_3_5_17.json']
    uriP = 'http://d-nb.info/gnd/11868499X'
    kindP = 'Person'
    uriGeon = 'http://sws.geonames.org/2761369/'

    def test_parse_person(self):
        print('Number of Persons: {}'.format(Person.objects.all().count()))
        o = GenericRDFParser(self.uriP, self.kindP)
        o2 = o.save()
        print('got following person: {}'.format(o2))
        self.assertEqual(o.objct.name, 'Eosander')
        self.assertEqual(o.objct.first_name, 'Johann Friedrich')

    def test_place_of_birth_death(self):
        o = GenericRDFParser(self.uriP, self.kindP)
        p2 = o.save()
        print(p2.start_date)
        print(p2.end_date)
        rel_type_birth = PersonPlaceRelation.objects.get(name='place of birth')
        rel_type_death = PersonPlaceRelation.objects.get(name='place of death')
        rel_birth = PersonPlace.objects.get(related_person=o.objct, relation_type=rel_type_birth)
        rel_death = PersonPlace.objects.get(related_person=o.objct, relation_type=rel_type_death)
        self.assertEqual(rel_birth.related_place.name, 'Stralsund')
        self.assertEqual(rel_death.related_place.name, 'Dresden')
        plc_list = Label.objects.filter(temp_entity=rel_death.related_place).values_list('label',flat=True)
        print(plc_list)
        self.assertTrue('Dressden' in plc_list)
        self.assertEqual(rel_death.related_place.lat, 13.738319)
        print('lat: {}, lng: {}'.format(rel_death.related_place.lat, rel_death.related_place.lng))

    def test_merge_places(self):
        txt = Text.objects.create(text='test text')
        src = Source.objects.create(orig_id=24, pubinfo='test pub')
        pp = Place(name="Wien")
        pp.source = src
        pp.save()
        pp.text.add(txt)
        rel_type_birth = PersonPlaceRelation.objects.create(name='place of birth')
        pers = Person.objects.create(name="tesdt", first_name="test3")
        rel_1 = PersonPlace.objects.create(related_person=pers, relation_type=rel_type_birth, related_place=pp)
        ow = GenericRDFParser(self.uriGeon, 'Place')
        print('name: {}, lat: {}, long: {}, labels: {}'.format(ow.objct.name, ow.objct.lat, ow.objct.lng, ' / '.join(
            Label.objects.filter(temp_entity=ow.objct).values_list('label', flat=True))))
        if ow.created:
            print('created triggered')
            ow.save()
        ow2 = ow.merge(pp)
        print(ow)
        print(ow2)
        self.assertEqual(pp.source.pubinfo, ow.objct.source.pubinfo)
        self.assertEqual(txt.text, ow.objct.text.all()[0].text)
        for x in PersonPlace.objects.all():
            self.assertEqual(x.related_place.pk, ow.objct.pk)


class RDFPlaceParserTestCase(TestCase):
    #fixtures = ['fixtures_3_5_17.json']
    kindP = 'Place'
    uriGeon = 'http://sws.geonames.org/2761369/'
    uri_gnd_geo = 'http://d-nb.info/gnd/4023118-5'

    def test_parse_place(self):
        o = GenericRDFParser(self.uriGeon, self.kindP)
        o2 = o.save()
        print('name: {}, lat: {}, long: {}, labels: {}, kind: {}'.format(o2.place.name,
                                                               o2.place.lat, o2.place.lng,
                                                               ' / '.join(Label.objects.filter(temp_entity=o2.place).values_list('label', flat=True)),
                                                                o2.kind.name))
        t = GenericRDFParser(self.uri_gnd_geo, self.kindP)
        t2 = t.save()
        print('name: {}, lat: {}, long: {}, labels: {}'.format(t2.name, t2.lat, t2.lng, ' / '.join(
            Label.objects.filter(temp_entity=t2).values_list('label', flat=True))))
        self.assertEqual(o2.lat, '48.20849')
        self.assertEqual(t2.lat, '053.583329')

    def test_merge_place(self):
        col = Collection.objects.create(name='test coll one')
        col2 = Collection.objects.create(name='test coll 2')
        w1 = Place.objects.create(name='Wien 21 test')
        w1.collection.add(col)
        w1.collection.add(col2)
        w1_label = Label.objects.create(label='Wien label 1', temp_entity=w1)
        w1_uri = Uri.objects.create(uri='http://test.uri.ac.at/orig:id', entity=w1)
        o = GenericRDFParser(self.uriGeon, self.kindP)
        o2 = o.save()
        o.merge(w1)
        for col in o2.collection.all():
            print('collection: {}'.format(col.name))
        self.assertEqual(Label.objects.filter(temp_entity=o2, label='Wien label 1').count(), 1)
        self.assertEqual(Uri.objects.filter(entity=o2, uri='http://test.uri.ac.at/orig:id').count(), 1)
        self.assertEqual(
            Label.objects.filter(label='Wien 21 test', temp_entity=o2, label_type__name='legacy name').count(), 1)
        self.assertEqual(o2.collection.all().count(), 3)

    def test_related_places(self):
        o = GenericRDFParser(self.uriGeon, self.kindP)
        o2 = o.save()
        print(o2)
        print('related_objects: {}'.format(o.related_objcts))
        print('number of placeplace: {}'.format(PlacePlace.objects.all().count()))
        pp = PlacePlace.objects.filter(Q(related_placeA=o2)|Q(related_placeB=o2))
        self.assertEqual(pp.count(), 2)

    def test_existing_place(self):
        c = GenericRDFParser('http://sws.geonames.org/2782113/', self.kindP)
        c.save()
        o = GenericRDFParser(self.uriGeon, self.kindP)
        o2 = o.save()
        Uri.objects.create(uri='http://test.at', entity=o2)
        pp = PlacePlace.objects.filter(Q(related_placeA=o2) | Q(related_placeB=o2))
        self.assertEqual(pp.count(), 2)
        for p in Place.objects.all():
            print('name: {}, uri: {}'.format(p.name, ' / '.join(p.uri_set.all().values_list('uri', flat=True))))
        cc = Place.objects.filter(uri__uri__icontains='http://sws.geonames.org/2782113').distinct()
        self.assertEqual(cc.count(), 1)
        i = GenericRDFParser(self.uriGeon, self.kindP)
        i2 = i.save()
        print(' / '.join(i.objct.uri_set.all().values_list('uri', flat=True)))
        print(i.objct)
        cc = Place.objects.filter(uri__uri=self.uriGeon).distinct()
        self.assertEqual(cc.count(), 1)




import django_tables2 as tables
from django_tables2.utils import A
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.utils.safestring import mark_safe

from .models import Person, Place, Institution, Event, Work

input_form = """
  <input type="checkbox" name="keep" value="{}" title="keep this"/> |
  <input type="checkbox" name="remove" value="{}" title="remove this"/>
"""


class MergeColumn(tables.Column):
    """ renders a column with to checkbox - used to select objects for merging """

    def __init__(self, *args, **kwargs):
        super(MergeColumn, self).__init__(*args, **kwargs)

    def render(self, value):
        return mark_safe(
            input_form.format(value, value)
        )


def get_entities_table(entity, edit_v, default_cols=['name', ]):

    class GenericEntitiesTable(tables.Table):
        if edit_v:
            name = tables.LinkColumn(
                'apis:apis_entities:generic_entities_edit_view',
                args=[entity.lower(), A('pk')]
            )
        else:
            name = tables.LinkColumn(
                'apis:apis_entities:generic_entities_detail_view',
                args=[entity.lower(), A('pk')]
            )
        export_formats = [
            'csv',
            'json',
            'xls',
            'xlsx',
        ]
        if 'merge' in default_cols:
            merge = MergeColumn(verbose_name='keep | remove', accessor='pk')
        if 'id' in default_cols:
            id = tables.LinkColumn()

        class Meta:
            model = ContentType.objects.get(
                app_label__startswith='apis_', model=entity.lower()).model_class()
            fields = default_cols
            attrs = {"class": "table table-hover table-striped table-condensed"}
    return GenericEntitiesTable


class PersonTable(tables.Table):
    name = tables.LinkColumn('apis:apis_entities:person_edit', args=[A('pk')])

    class Meta:
        model = Person
        fields = ['name', 'first_name', 'start_date', 'end_date']
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed"}


class PlaceTable(tables.Table):
    name = tables.LinkColumn('apis:apis_entities:place_edit', args=[A('pk')])

    class Meta:
        model = Place
        fields = ['name', 'status', 'lng', 'lat']
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed"}


class InstitutionTable(tables.Table):
    name = tables.LinkColumn('apis:apis_entities:institution_edit', args=[A('pk')])

    class Meta:
        model = Institution
        fields = ['name', 'start_date', 'end_date', 'kind']
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed"}


class EventTable(tables.Table):
    name = tables.LinkColumn('apis:apis_entities:event_edit', args=[A('pk')])

    class Meta:
        model = Event
        fields = ['name', 'start_date', 'end_date', 'kind']
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed"}


class WorkTable(tables.Table):
    name = tables.LinkColumn('apis:apis_entities:work_edit', args=[A('pk')])

    class Meta:
        model = Work
        fields = ['name', 'start_date', 'end_date', 'kind']
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed"}



from rest_framework import viewsets
from .serializers import (
    InstitutionInstitutionSerializer, PersonInstitutionSerializer,
    PersonPlaceSerializer, PersonPersonSerializer, PersonEventSerializer, PersonWorkSerializer,
    InstitutionPlaceSerializer, InstitutionEventSerializer, InstitutionWorkSerializer, PlaceEventSerializer,
    PlaceWorkSerializer, PlacePlaceSerializer, EventWorkSerializer, EventEventSerializer, WorkWorkSerializer)
from .models import (InstitutionInstitution, PersonInstitution, PersonPlace, PersonPerson, PersonEvent, PersonWork,
                     InstitutionPlace, InstitutionEvent, InstitutionWork, EventWork, EventEvent, PlaceEvent, PlaceWork,
                     PlacePlace, WorkWork)
from apis_core.apis_entities.api_views import StandardResultsSetPagination


class InstitutionInstitutionViewSet(viewsets.ModelViewSet):
    queryset = InstitutionInstitution.objects.all()
    serializer_class = InstitutionInstitutionSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_institutionA__name', 'related_institutionB__name')


class PersonInstitutionViewSet(viewsets.ModelViewSet):
    queryset = PersonInstitution.objects.all()
    serializer_class = PersonInstitutionSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_person__name', 'related_institution__name')


class PersonPlaceViewSet(viewsets.ModelViewSet):
    queryset = PersonPlace.objects.all()
    serializer_class = PersonPlaceSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_place__name', 'related_person__name')


class PersonPersonViewSet(viewsets.ModelViewSet):
    queryset = PersonPerson.objects.all()
    serializer_class = PersonPersonSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_personA__name', 'related_personB__name')


class PersonEventViewSet(viewsets.ModelViewSet):
    queryset = PersonEvent.objects.all()
    serializer_class = PersonEventSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_person__name', 'related_event__name')


class PersonWorkViewSet(viewsets.ModelViewSet):
    queryset = PersonWork.objects.all()
    serializer_class = PersonWorkSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_person__name', 'related_work__name')


class InstitutionPlaceViewSet(viewsets.ModelViewSet):
    queryset = InstitutionPlace.objects.all()
    serializer_class = InstitutionPlaceSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_institution__name', 'related_place__name')


class InstitutionEventViewSet(viewsets.ModelViewSet):
    queryset = InstitutionEvent.objects.all()
    serializer_class = InstitutionEventSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_institution__name', 'related_event__name')


class InstitutionWorkViewSet(viewsets.ModelViewSet):
    queryset = InstitutionWork.objects.all()
    serializer_class = InstitutionWorkSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_institution__name', 'related_work__name')


class EventWorkViewSet(viewsets.ModelViewSet):
    queryset = EventWork.objects.all()
    serializer_class = EventWorkSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_event__name', 'related_work__name')


class EventEventViewSet(viewsets.ModelViewSet):
    queryset = EventEvent.objects.all()
    serializer_class = EventEventSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_eventA__name', 'related_eventB__name')


class PlaceEventViewSet(viewsets.ModelViewSet):
    queryset = PlaceEvent.objects.all()
    serializer_class = PlaceEventSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_place__name', 'related_event__name')


class PlaceWorkViewSet(viewsets.ModelViewSet):
    queryset = PlaceWork.objects.all()
    serializer_class = PlaceWorkSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_place__name', 'related_work__name')


class PlacePlaceViewSet(viewsets.ModelViewSet):
    queryset = PlacePlace.objects.all()
    serializer_class = PlacePlaceSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_placeA__name', 'related_placeB__name')


class WorkWorkViewSet(viewsets.ModelViewSet):
    queryset = WorkWork.objects.all()
    serializer_class = WorkWorkSerializer
    pagination_class = StandardResultsSetPagination
    depth = 2
    filter_fields = ('related_workA__name', 'related_workB__name')



from rest_framework import serializers
from .models import (InstitutionInstitution, PersonInstitution, PersonPlace, PersonPerson,
                     PersonEvent, PersonWork, InstitutionPlace, InstitutionEvent, InstitutionWork,
                     PlaceEvent, PlaceWork,PlacePlace, EventWork, EventEvent, WorkWork)


class InstitutionInstitutionSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionInstitution


class PersonInstitutionSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonInstitution


class PersonPlaceSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonPlace


class PersonPersonSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonPerson


class PersonEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonEvent


class PersonWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PersonWork


class InstitutionPlaceSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionPlace


class InstitutionEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionEvent


class InstitutionWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = InstitutionWork


class PlaceEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlaceEvent


class PlaceWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlaceWork


class PlacePlaceSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = PlacePlace


class EventWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = EventWork


class EventEventSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = EventEvent


class WorkWorkSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        fields = '__all__'
        model = WorkWork



from django.contrib.contenttypes.models import ContentType
from django.urls import reverse
from rest_framework import serializers
from django.db.models.query import QuerySet
from apis_core.apis_labels.serializers import LabelSerializer
from apis_core.apis_vocabularies.models import ProfessionType


import re


class CollectionSerializer(serializers.Serializer): 
    id = serializers.IntegerField()
    name = serializers.CharField()


class ProfessionTypeSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = ProfessionType
        fields = ('id', 'name', 'label')


class EntityUriSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    uri = serializers.URLField()


class EntitySerializer(serializers.Serializer):
    id = serializers.IntegerField()
    name = serializers.CharField()
    start_date = serializers.DateField()
    end_date = serializers.DateField()
    uris = EntityUriSerializer(source="uri_set", many=True)
    labels = LabelSerializer(source="label_set", many=True)

    def add_relations(self, obj):
        res = {}
        mk = obj.__class__.__name__
        for rel in ContentType.objects.filter(
                app_label='apis_relations', model__icontains=mk.lower()):
            mk2 = re.match(r'{}([A-Za-z]+)'.format(mk.lower()), rel.model)
            if not mk2:
                mk2 = re.match(r'([A-Za-z]+{})'.format(mk.lower()), rel.model)
            res['{}s'.format(mk2.group(1))] = []
            if mk2.group(1).lower() != mk.lower():
                for rel2 in getattr(obj, '{}_set'.format(rel.model)).all():
                    res['{}s'.format(mk2.group(1))].append(RelationEntitySerializer(rel2, own_class=mk, read_only=True, context=self.context).data)
            else:
                for t in ['A', 'B']:
                    for rel2 in getattr(obj, 'related_{}{}'.format(mk.lower(), t)).all():
                        if t == 'A':
                            ok = '{}A'.format(mk.lower())
                        else:
                            ok = '{}B'.format(mk.lower())
                        res['{}s'.format(mk2.group(1))].append(RelationEntitySerializer(rel2, own_class=ok, read_only=True, context=self.context).data)
        return res

    def add_entity_type(self, obj):
        return str(obj.__class__.__name__)

    def __init__(self, *args, depth_ent=1, **kwargs):
        super(EntitySerializer, self).__init__(*args, **kwargs)
        if type(self.instance) == QuerySet:
            inst = self.instance[0]
        else:
            inst = self.instance
        for f in inst._meta.fields:
            field_name = re.search(r'([A-Za-z]+)\'>', str(f.__class__)).group(1)
            if field_name in [
                'CharField', 'DateField', 'DateTimeField', 'IntegerField', 'FloatField'
            ]:
                self.fields[f.name] = getattr(serializers, field_name)()
        for f in inst._meta.many_to_many:
            if f.name == 'profession':
                self.fields['profession'] = ProfessionTypeSerializer(many=True)
            elif f.name == 'collection':
                self.fields['collection'] = CollectionSerializer(many=True)
        self.fields['entity_type'] = serializers.SerializerMethodField(
            method_name="add_entity_type"
        )
        if depth_ent == 1:
            self.fields['relations'] = serializers.SerializerMethodField(
                method_name="add_relations"
            )


class RelationEntitySerializer(serializers.Serializer):
    id = serializers.IntegerField()
    relation_type = serializers.SerializerMethodField(method_name="add_relation_label")

    def add_entity(self, obj):
        return EntitySerializer(getattr(obj, "related_{}".format(self.entity_type)), depth_ent=0).data
    
    def add_relation_label(self, obj):
        cm = obj.__class__.__name__
        cm_match_1 = re.match(r'{}.*'.format(self.own_class), cm)
        cm_match_2 = re.match(r'.*{}'.format(self.own_class), cm)
        res_1 = dict()
        request_1 = self.context.get('request', None)
        if request_1 is not None:
            res_1['url'] =  self.context['request'].build_absolute_uri(reverse("apis_core:{}relation-detail".format(cm).lower(), kwargs={'pk':obj.relation_type.pk})) 
        else:
            res_1['url'] =  reverse("apis_core:{}relation-detail".format(cm).lower(), kwargs={'pk':obj.relation_type.pk})
        if cm_match_1:
            res_1['label'] =  obj.relation_type.label
        elif cm_match_2:
            res_1['label'] = obj.relation_type.label_reverse
        else:
            res_1['label'] =  "({})".format(obj.relation_type.label)
        return res_1

    def __init__(self, *args, own_class=None, **kwargs):
        super(RelationEntitySerializer, self).__init__(*args, **kwargs)
        self.own_class = own_class
        if self.instance is not None:
            for f in self.instance._meta.fields:
                if f.name.startswith('related_'):
                    mk2 = f.name.replace('related_', '')

                    if mk2.lower() != own_class.lower():
                        self.entity_type = mk2
                        if re.match(r'.*[A-B]$', mk2):
                            mk2_ = mk2[:-1]
                        else:
                            mk2_ = mk2
                        self.fields['{}'.format(mk2_)] = serializers.SerializerMethodField(
                            method_name="add_entity"
                        )



from django.conf.urls import url

from apis_core.apis_entities.autocomplete3 import GenericVocabulariesAutocomplete

app_name = 'apis_vocabularies'

urlpatterns = [
    url(r'^autocomplete/(?P<vocab>[a-zA-Z0-9-]+)/(?P<direct>[a-zA-Z0-9-]+)/$',
        GenericVocabulariesAutocomplete.as_view(),
        name='generic_vocabularies_autocomplete'),
]



from rest_framework import renderers
from apis_core.apis_tei.tei import TeiEntCreator
from rdflib import Namespace, Graph, URIRef, RDF, BNode, Literal, RDFS, XSD


class EntityToTEI(renderers.BaseRenderer):

    media_type = "text/xml"
    format = 'tei'

    def render(self, data, media_type=None, renderer_context=None):
        tei_doc = TeiEntCreator(data)
        return tei_doc.serialize_full_doc()


class EntityToCIDOC(renderers.BaseRenderer):

    media_type = "text/rdf+xml"
    format = 'rdf+xml'
    
    def render(self, data, media_type=None, renderer_context=None):
        cidoc = Namespace('http://www.cidoc-crm.org/cidoc-crm/')
        geo = Namespace('http://www.w3.org/2003/01/geo/wgs84_pos#')
        g = Graph()
        k_uri = URIRef('http://127.0.0.1:8000/apis/api2/entity/{}'.format(data['id']))
        g.add( (k_uri, RDF.type, cidoc.P21) )
        b_app = BNode()
        g.add( (k_uri, cidoc.P131, b_app) )
        g.add( (b_app, RDF.type, cidoc.E82) )
        g.add( (b_app, RDFS.label, Literal('{} {}'.format(data['first_name'], data['name']),  lang='de')) )
        b_birth = BNode()
        g.add( (b_birth, RDF.type, cidoc.E67) )
        g.add( (b_birth, cidoc.P98, k_uri) )
        place_of_birth = URIRef('http://127.0.0.1:8000/apis/api2/entity/{}'.format(data['relations']['places'][0]['id']))
        g.add( (b_birth, cidoc.P7, place_of_birth) )
        g.add( (place_of_birth, RDF.type, cidoc.E53) )
        g.add( (place_of_birth, RDFS.label, Literal(data['relations']['places'][0]['place']['name'], lang='de'))) 
        
        b_place_of_birth_app2 = BNode()
        g.add( (place_of_birth, cidoc.P87, b_place_of_birth_app2) )
        g.add(( b_place_of_birth_app2, RDF.type, cidoc.E47 ))
        g.add( (b_place_of_birth_app2, geo.lat, Literal(data['relations']['places'][0]['place']['lat'])) )
        g.add( (b_place_of_birth_app2, geo.long, Literal(data['relations']['places'][0]['place']['lng'])) )

        b_date_of_birth = BNode()
        g.add( (b_date_of_birth, RDF.type, cidoc.E50) )
        g.add( (b_date_of_birth, cidoc.P114, Literal(data['start_date'], datatype=XSD.date)) )

        return g.serialize(format='xml')



from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import UserPassesTestMixin
from django.db.models import Q
from django.http import HttpResponse
from django.template.loader import select_template
from django.utils.decorators import method_decorator
from django.views import View
from django.views.generic.detail import DetailView
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404
from django_tables2 import RequestConfig
from django.conf import settings

from apis_core.helper_functions.utils import access_for_all

from .views import get_highlighted_texts
from .models import Work
from apis_core.apis_labels.models import Label
from apis_core.apis_metainfo.models import Uri
from apis_core.apis_relations.tables import (
    get_generic_relations_table, EntityLabelTable, EntityDetailViewLabelTable
)


class GenericEntitiesDetailView(UserPassesTestMixin, View):

    login_url = '/accounts/login/'

    def test_func(self):
        access = access_for_all(self, viewtype="detail")
        return access

    def get(self, request, *args, **kwargs):
        entity = kwargs['entity'].lower()
        pk = kwargs['pk']
        entity_model = ContentType.objects.get(
            app_label='apis_entities', model=entity).model_class()
        instance = get_object_or_404(entity_model, pk=pk)
        relations = ContentType.objects.filter(app_label='apis_relations', model__icontains=entity)
        side_bar = []
        for rel in relations:
            print(str(rel))
            match = str(rel).split()
            prefix = "{}{}-".format(match[0].title()[:2], match[1].title()[:2])
            table = get_generic_relations_table(''.join(match), entity, detail=True)
            title_card = ''
            if match[0] == match[1]:
                title_card = entity.title()
                dict_1 = {'related_' + entity.lower() + 'A': instance}
                dict_2 = {'related_' + entity.lower() + 'B': instance}
                if 'apis_highlighter' in settings.INSTALLED_APPS:
                    object_pre = rel.model_class().annotation_links.filter_ann_proj(request=request).filter(
                        Q(**dict_1) | Q(**dict_2))
                else:
                    object_pre = rel.model_class().objects.filter(
                        Q(**dict_1) | Q(**dict_2))
                objects = []
                for x in object_pre:
                    objects.append(x.get_table_dict(instance))
            else:
                if match[0].lower() == entity.lower():
                    title_card = match[1].title()
                else:
                    title_card = match[0].title()
                dict_1 = {'related_' + entity.lower(): instance}
                if 'apis_highlighter' in settings.INSTALLED_APPS:
                    objects = list(rel.model_class()
                                   .annotation_links.filter_ann_proj(request=request)
                                   .filter(**dict_1))
                else:
                    objects = list(rel.model_class().objects.filter(**dict_1))
            tb_object = table(objects, prefix=prefix)
            tb_object_open = request.GET.get(prefix + 'page', None)
            RequestConfig(request, paginate={"per_page": 10}).configure(tb_object)
            side_bar.append(
                (title_card, tb_object, ''.join([x.title() for x in match]), tb_object_open)
            )
        object_lod = Uri.objects.filter(entity=instance)
        object_texts, ann_proj_form = get_highlighted_texts(request, instance)
        object_labels = Label.objects.filter(temp_entity=instance)
        tb_label = EntityDetailViewLabelTable(object_labels, prefix=entity.title()[:2]+'L-')
        tb_label_open = request.GET.get('PL-page', None)
        side_bar.append(('Label', tb_label, 'PersonLabel', tb_label_open))
        RequestConfig(request, paginate={"per_page": 10}).configure(tb_label)
        template = select_template([
            'apis_entities/detail_views/{}_detail_generic.html'.format(entity),
            'apis_entities/detail_views/entity_detail_generic.html'
            ])
        tei = getattr(settings, "APIS_TEI_TEXTS", [])
        if tei:
            tei = set(tei) & set([x.kind.name for x in instance.text.all()])
        ceteicean_css = getattr(settings, "APIS_CETEICEAN_CSS", None)
        ceteicean_js = getattr(settings, "APIS_CETEICEAN_JS", None)
        openseadragon_js = getattr(settings, "APIS_OSD_JS", None)
        openseadragon_img = getattr(settings, "APIS_OSD_IMG_PREFIX", None)
        iiif_field = getattr(settings, "APIS_IIIF_WORK_KIND", None)
        if iiif_field:
            try:
                if "{}".format(instance.kind) == "{}".format(iiif_field):
                    iiif = True
                else:
                    iiif = False
            except AttributeError:
                iiif = False
        else:
            iiif = False
        iiif_server = getattr(settings, "APIS_IIIF_SERVER", None)
        iiif_info_json = instance.name
        return HttpResponse(template.render(
            request=request, context={
                'entity_type': entity,
                'object': instance,
                'right_card': side_bar,
                'object_texts': object_texts,
                'object_lod': object_lod,
                'tei': tei,
                'ceteicean_css': ceteicean_css,
                'ceteicean_js': ceteicean_js,
                'iiif': iiif,
                'openseadragon_js': openseadragon_js,
                'openseadragon_img': openseadragon_img,
                'iiif_field': iiif_field,
                'iiif_info_json': iiif_info_json,
                'iiif_server': iiif_server,
                }
            ))


class WorkDetailView(DetailView):
    model = Work
    template_name = 'apis_entities/detail_views/work_detail.html'

    def get_context_data(self, **kwargs):
        context = super(WorkDetailView, self).get_context_data(**kwargs)
        return context



import django_filters
from dal import autocomplete
from django.forms import ModelMultipleChoiceField
from django.urls import reverse

from .models import Person, Place, Institution, Event, Work
from django.db.models import Q
from django.conf import settings
from django.contrib.contenttypes.models import ContentType


django_filters.filters.LOOKUP_TYPES = [
    ('', '---------'),
    ('icontains', 'Contains (case insensitive)'),
    ('exact', 'Is equal to'),
    ('iexact', 'Is equal to (case insensitive)'),
    ('not_exact', 'Is not equal to'),
    ('lt', 'Lesser than/before'),
    ('gt', 'Greater than/after'),
    ('gte', 'Greater than or equal to'),
    ('lte', 'Lesser than or equal to'),
    ('startswith', 'Starts with'),
    ('endswith', 'Ends with'),
    ('contains', 'Contains'),
    ('not_contains', 'Does not contain'),
]


def get_generic_list_filter(entity):
    class GenericListFilter(django_filters.FilterSet):
        def name_label_filter(self, queryset, name, value):
            """
            Filter for including the alternative names in the names search. The types of labels included in the query are
            currently hardcoded in a list.

            :param queryset: queryset that the filters are applied on
            :param name: name of the attribute to filter on (not used as label types are hardcoded)
            :param value: value for the filter
            :return: filtered queryset
            """
            alternate_names = getattr(settings, "APIS_ALTERNATE_NAMES", ['alternative name'])
            res = []
            orig_value = value
            for n in ['name', 'label__label']:
                value = orig_value
                f = '{}__'.format(n)
                if value.startswith('"') and value.endswith('"'):
                    value = value[1:-1]
                else:
                    f += 'i'
                if value.startswith('*') and value.endswith('*'):
                    f += 'contains'
                    value = value[1:-1]
                elif value.startswith('*'):
                    f += 'endswith'
                    value = value[1:]
                elif value.endswith('*'):
                    f += 'startswith'
                    value = value[:-1]
                else:
                    f += 'exact'
                if n == 'label__label':
                    res.append(Q(**{f: value, 'label__label_type__name__in': alternate_names}))
                else:
                    res.append(Q(**{f: value}))
            return queryset.filter(res[0] | res[1] ).distinct()

        def wildcard_filter(self, queryset, name, value):
            f = '{}__'.format(name)
            if value.startswith('"') and value.endswith('"'):
                value = value[1:-1]
            else:
                f += 'i'
            if value.startswith('*') and value.endswith('*'):
                f += 'contains'
                value = value[1:-1]
            elif value.startswith('*'):
                f += 'endswith'
                value = value[1:]
            elif value.endswith('*'):
                f += 'startswith'
                value = value[:-1]
            else:
                f += 'exact'
            return queryset.filter(**{f: value})

        class Meta:
            model = ContentType.objects.get(
                app_label__startswith='apis_', model=entity.lower()).model_class()

            try:
                if 'list_filters' in settings.APIS_ENTITIES[entity.title()].keys():
                    fields = [
                        x[0] for x in settings.APIS_ENTITIES[entity.title()]['list_filters']
                    ]
            except KeyError:
                exclude = ('MetaInfo',
                           'collection',
                           'references',
                           'notes',
                           'review',
                           'start_date_written',
                           'end_date_written',
                           'source',
                           'tempentityclass_ptr',
                           'id',
                           'annotation_set_relation',
                           'text')

        def __init__(self, *args, **kwargs):
            attrs = {'data-placeholder': 'Type to get suggestions',
                     'data-minimum-input-length': 3,
                     'data-html': True}
            super(GenericListFilter, self).__init__(*args, **kwargs)
            try:
                if 'list_filters' in settings.APIS_ENTITIES[entity.title()].keys():
                    for f in settings.APIS_ENTITIES[entity.title()]['list_filters']:
                        for ff in f[1].keys():
                            setattr(self.filters[f[0]], ff, f[1][ff])
            except KeyError:
                pass
            for f in self.filters.keys():
                if type(self.filters[f].field) == ModelMultipleChoiceField:
                    v_name_p = str(self.filters[f].queryset.model.__name__)
                    if ContentType.objects.get(app_label__in=[
                        'apis_entities', 'apis_metainfo', 'apis_relations',
                        'apis_vocabularies', 'apis_labels'
                    ], model=v_name_p.lower()).app_label.lower() == 'vocabularies':
                        self.filters[f].field.widget = autocomplete.Select2Multiple(
                            url=reverse('vocabularies:generic_vocabularies_autocomplete', kwargs={
                                'vocab': v_name_p.lower(),
                                'direct': 'normal'
                            }),
                            attrs=attrs)

    return GenericListFilter


class PersonListFilter(django_filters.FilterSet):
    FILTER_CHOICES = (('', 'any'), ('male', 'male'), ('female', 'female'))
    name = django_filters.CharFilter(method='name_label_filter', label='Name')
    first_name = django_filters.CharFilter(lookup_expr='icontains', label='First Name')
    gender = django_filters.ChoiceFilter(choices=FILTER_CHOICES)
    start_date = django_filters.DateFilter(label='Date of birth') # Todo: add a datefilter that allows to filter for ranges
    end_date = django_filters.DateFilter(label='Date of death')
    profession__name = django_filters.CharFilter(lookup_expr='icontains', label='Profession')

    class Meta:
        model = Person
        fields = ['name', 'first_name', 'gender', 'start_date', 'end_date', 'profession__name', 'collection']

    def name_label_filter(self, queryset, name, value):
        """
        Filter for including the alternative names in the names search. The types of labels included in the query are
        currently hardcoded in a list.

        :param queryset: queryset that the filters are applied on
        :param name: name of the attribute to filter on (not used as label types are hardcoded)
        :param value: value for the filter
        :return: filtered queryset
        """
        return queryset.filter(Q(name__icontains=value)|
                               Q(label__label__icontains=value, label__label_type__name__in=alternate_names)).distinct()


class PlaceListFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr='icontains', label='Name')
    lng = django_filters.NumberFilter(label='Longitude') # Todo: add filters that allow ranges
    lat = django_filters.NumberFilter(label='Latitude')
    status = django_filters.CharFilter(lookup_expr='icontains')

    class Meta:
        model = Place
        fields = ['name', 'status', 'lng', 'lat', 'collection']


class InstitutionListFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr='icontains', label='Name')
    start_date = django_filters.DateFilter(label='Date of foundation')
    end_date = django_filters.DateFilter(label='Date of closing')

    class Meta:
        model = Institution
        fields = ['name', 'start_date', 'end_date', 'collection']


class EventListFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr='icontains', label='Name')
    start_date = django_filters.DateFilter(label='Start date')
    end_date = django_filters.DateFilter(label='End date')
    kind__name = django_filters.CharFilter(lookup_expr='icontains', label='Kind')

    class Meta:
        model = Event
        fields = ['name', 'start_date', 'end_date', 'kind__name', 'collection']


class WorkListFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr='icontains', label='Name')
    start_date = django_filters.DateFilter(label='Start date')
    end_date = django_filters.DateFilter(label='End date')
    kind__name = django_filters.CharFilter(lookup_expr='icontains', label='Kind')

    class Meta:
        model = Work
        fields = ['name', 'start_date', 'end_date', 'kind__name', 'collection']



#!/usr/bin/python
# -*- coding: utf-8 -*-
from datetime import datetime

from django import forms
import autocomplete_light.shortcuts as al
from django.utils.translation import ugettext_lazy as _
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout
from crispy_forms.bootstrap import Accordion, AccordionGroup
from django.contrib.contenttypes.models import ContentType

from .models import (PersonPlace, PersonPerson, PersonInstitution,
                     InstitutionPlace, InstitutionInstitution, PersonEvent, InstitutionEvent,
                     PlaceEvent, PersonWork, InstitutionWork, PlaceWork, EventWork, PlacePlace)
from apis_core.entities.models import Place, Institution, Person, Event, Work
from apis_core.metainfo.models import Uri, Text
from apis_core.vocabularies.models import (PersonInstitutionRelation, PersonPlaceRelation, InstitutionPlaceRelation,
                                 PersonEventRelation, InstitutionEventRelation, PlaceEventRelation,
                                 InstitutionInstitutionRelation, PersonPersonRelation)
from apis_core.labels.models import Label
from highlighter.models import Annotation
from apis_core.helper_functions.RDFparsers import GenericRDFParser


##############################################
# Generic
##############################################

class EntityLabelForm(forms.ModelForm):

    class Meta:
        model = Label
        fields = ['label', 'isoCode_639_3', 'label_type']

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = Label.objects.get(pk=instance)
            x.label = cd['label']
            x.isoCode_639_3 = cd['isoCode_639_3']
            x.label_type = cd['label_type']
        else:
            x = super(EntityLabelForm, self).save(commit=False)
            x.temp_entity = site_instance
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(EntityLabelForm, self).__init__(*args, **kwargs)
        self.fields['label'].required = True
        self.fields['label_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'EntityLabelForm'
        self.helper.form_tag = False

##############################################
# Person
##############################################


class PersonPlaceForm(forms.ModelForm):
    place = forms.CharField(label='Ort', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    person = forms.CharField(label='Person', widget=al.TextWidget('PersonAutocomplete'))
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = PersonPlace
        fields = ['relation_type', 'place', 'place_uri', 'person', 'person_uri', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type')
            }
        widgets = {
        'relation_type': al.ChoiceWidget('VCPersonPlaceAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PersonPlace.objects.get(pk=str(instance))
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(PersonPlaceForm, self).save(commit=False)
        if 'place' in cd.keys():
            x.related_person = site_instance
            place = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
            x.related_place = place
        elif 'person' in cd.keys():
            x.related_place = site_instance
            person = Person.get_or_create_uri(cd['person_uri'])
            if not person:
                person = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()
            x.related_person = person
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        if not hasattr(self, 'request'):
            self.request = kwargs.pop('request', False)
        super(PersonPlaceForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PersonPlaceForm'
        self.helper.form_tag = False
        self.helper.layout = Layout(
                'relation_type',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="persPlace_notes_refs")))
        if entity_type == Place:
            choices = []
            self.fields.pop('place')
            self.fields.pop('place_uri')
            for x in PersonPlaceRelation.objects.all():
                choices.append((x.pk, x.name_reverse))
            self.fields['relation_type'].choices = choices
            if instance and instance.id:
                self.fields['person'].initial = instance.related_person
                self.fields['person_uri'].initial = Uri.objects.filter(entity=instance.related_person)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPersonPlaceReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonPlaceReverseAutocomplete')
            self.helper.layout.insert(1, 'person')
            self.helper.layout.insert(2, 'person_uri')            
        else:
            self.fields.pop('person')
            self.fields.pop('person_uri')
            if instance and instance.id:
                self.fields['place'].initial = instance.related_place.name
                self.fields['place_uri'].initial = Uri.objects.filter(entity=instance.related_place)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPersonPlaceReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout.insert(1, 'place')
            self.helper.layout.insert(2, 'place_uri')


class PersonPersonForm(forms.ModelForm):
    person = forms.CharField(label='Person', widget=al.TextWidget('PersonAutocomplete'))
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = PersonPerson
        fields = ['relation_type', 'person', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Art'),
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCPersonPersonAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PersonPerson.objects.select_related(
                'related_personA__tempentityclass_ptr__uri',
                'related_personB__tempentityclass_ptr__uri').get(pk=instance)
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.relation_type = cd['relation_type']
            x.notes = cd['notes']
            x.references = cd['references']
            if site_instance == x.related_personB:
                if x.related_personA.uri_set.filter(uri=cd['person_uri']).count() == 0:
                    personA = Person.get_or_create_uri(cd['person_uri'])
                    if not personA:
                        personA = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()
                    x.related_personA = personA
        else:
            x = super(PersonPersonForm, self).save(commit=False)
            x.related_personA = site_instance
            personB = Person.get_or_create_uri(cd['person_uri'])
            if not personB:
                personB = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()  # TODO: Function needs to be replaced with class
            x.related_personB = personB
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        if not hasattr(self, 'request'):
            self.request = kwargs.pop('request', False)
        super(PersonPersonForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            if siteID:
                siteID = Person.objects.get(pk=siteID)
                if siteID == instance.related_personB:
                    rel_pers = instance.related_personA
                    self.fields['relation_type'].widget = al.ChoiceWidget(
                        'VCPersonPersonReverseAutocomplete',
                        extra_context={'values': [instance.relation_type.pk],
                                       'choices': [instance.relation_type.label_reverse]})
                else:
                    rel_pers = instance.related_personB
            self.fields['person'].initial = rel_pers
            self.fields['person_uri'].initial = Uri.objects.filter(entity=rel_pers)[0]
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PersonPersonForm'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            'relation_type',
            'person',
            'person_uri',
            'start_date_written',
            'end_date_written',
            Accordion(
                AccordionGroup(
                    'Notes and References',
                    'notes',
                    'references',
                    active=False,
                    css_id="persPers_notes_refs")))


class PersonInstitutionForm(forms.ModelForm):
    institution = forms.CharField(
        label='Institution',
        widget=al.TextWidget('InstitutionAutocomplete'))
    institution_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    person = forms.CharField(label='Person', widget=al.TextWidget('PersonAutocomplete'))
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    #relation_type = forms.CharField(label='relation Type', widget=al.TextWidget('VCPersonInstitutionAutocomplete'))

    class Meta:
        model = PersonInstitution
        fields = ['relation_type', 'person', 'institution', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type'),
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCPersonInstitutionAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PersonInstitution.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(PersonInstitutionForm, self).save(commit=False)
        if 'institution' in cd.keys():
            x.related_person = site_instance
            institution = Institution.get_or_create_uri(cd['institution_uri'])
            if not institution:
                institution = GenericRDFParser(cd['institution_uri'], 'Institution').get_or_create()
            x.related_institution = institution
        elif 'person' in cd.keys():
            x.related_institution = site_instance
            person = Person.get_or_create_uri(cd['person_uri'])
            if not person:
                person = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()
            x.related_person = person
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        if not hasattr(self, 'request'):
            self.request = kwargs.pop('request', False)
        super(PersonInstitutionForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            self.fields['institution'].initial = instance.related_institution.name
            self.fields['institution_uri'].initial = Uri.objects.filter(entity=instance.related_institution)[0]
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PersonInstitutionForm'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            'relation_type',
            'start_date_written',
            'end_date_written',
            Accordion(
                AccordionGroup(
                    'Notes and References',
                    'notes',
                    'references',
                    active=False,
                    css_id="persInst_notes_refs")))
        if entity_type == Institution:
            choices = []
            for x in PersonInstitutionRelation.objects.all():
                choices.append((x.pk, x.name_reverse))

            self.fields.pop('institution')
            self.fields.pop('institution_uri')
            if instance and instance.id:
                self.fields['person'].initial = instance.related_person
                self.fields['person_uri'].initial = Uri.objects.filter(entity=instance.related_person)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonInstitutionReverseAutocomplete',
                                                                      extra_context={
                                                                          'values': [instance.relation_type.pk],
                                                                          'choices': [
                                                                              instance.relation_type.label_reverse]}
                                                                      )
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonInstitutionReverseAutocomplete')
            self.helper.layout.insert(1, 'person')
            self.helper.layout.insert(2, 'person_uri')
        else:
            self.fields.pop('person')
            self.fields.pop('person_uri')
            if instance and instance.id:
                self.fields['institution'].initial = instance.related_institution.name
                self.fields['institution_uri'].initial = Uri.objects.filter(entity=instance.related_institution)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonInstitutionReverseAutocomplete',
                                                                      extra_context={
                                                                          'values': [instance.relation_type.pk],
                                                                          'choices': [
                                                                              instance.relation_type.label]}
                                                                      )
            self.helper.layout.insert(1, 'institution')
            self.helper.layout.insert(2, 'institution_uri')


class PersonEventForm(forms.ModelForm):
    event = forms.CharField(
        label='Event',
        widget=al.TextWidget('EventAutocomplete'))
    event_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    person = forms.CharField(label='Person', widget=al.TextWidget('PersonAutocomplete'))
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    #relation_type = forms.CharField(label='relation Type', widget=al.TextWidget('VCPersonInstitutionAutocomplete'))

    class Meta:
        model = PersonEvent
        fields = ['relation_type', 'person', 'event', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type'),
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCPersonEventAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PersonEvent.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(PersonEventForm, self).save(commit=False)
        if 'event' in cd.keys():
            x.related_person = site_instance
            event = Event.get_or_create_uri(cd['event_uri'])
            if not event:
                event = GenericRDFParser(cd['event_uri'], 'Event').get_or_create()
            x.related_event = event
        elif 'person' in cd.keys():
            x.related_event = site_instance
            person = Person.get_or_create_uri(cd['person_uri'])
            if not person:
                person = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()
            x.related_person = person
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        if not hasattr(self, 'request'):
            self.request = kwargs.pop('request', False)
        super(PersonEventForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            self.fields['event'].initial = instance.related_event.name
            self.fields['event_uri'].initial = Uri.objects.filter(entity=instance.related_event)[0]
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PersonEventForm'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            'relation_type',
            'start_date_written',
            'end_date_written',
            Accordion(
                AccordionGroup(
                    'Notes and References',
                    'notes',
                    'references',
                    active=False,
                    css_id="persEvent_notes_refs")))
        if entity_type == Event:
            choices = []
            for x in PersonEventRelation.objects.all():
                choices.append((x.pk, x.name_reverse))
            self.fields['relation_type'].choices = choices
            self.fields.pop('event')
            self.fields.pop('event_uri')
            if instance and instance.id:
                self.fields['person'].initial = instance.related_person
                self.fields['person_uri'].initial = Uri.objects.filter(entity=instance.related_person)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPersonEventReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonEventReverseAutocomplete')
            self.helper.layout.insert(1, 'person')
            self.helper.layout.insert(2, 'person_uri')
        else:
            self.fields.pop('person')
            self.fields.pop('person_uri')
            if instance and instance.id:
                self.fields['event'].initial = instance.related_event.name
                self.fields['event_uri'].initial = Uri.objects.filter(entity=instance.related_event)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPersonEventReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout.insert(1, 'event')
            self.helper.layout.insert(2, 'event_uri')


class PersonWorkForm(forms.ModelForm):
    work = forms.CharField(
        label='Work',
        widget=al.TextWidget('WorkAutocomplete'))
    work_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    person = forms.CharField(label='Person', widget=al.TextWidget('PersonAutocomplete'))
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    #relation_type = forms.CharField(label='relation Type', widget=al.TextWidget('VCPersonInstitutionAutocomplete'))

    class Meta:
        model = PersonWork
        fields = ['relation_type', 'person', 'work', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type'),
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCPersonWorkAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PersonWork.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(PersonWorkForm, self).save(commit=False)
        if 'work' in cd.keys():
            x.related_person = site_instance
            work = Work.get_or_create_uri(cd['work_uri'])
            if not work:
                work = GenericRDFParser(cd['work_uri'], 'Work')  # TODO: Create work entries for GND parser
            x.related_work = work
        elif 'person' in cd.keys():
            x.related_work = site_instance
            person = Person.get_or_create_uri(cd['person_uri'])
            if not person:
                person = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()
            x.related_person = person
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        if not hasattr(self, 'request'):
            self.request = kwargs.pop('request', False)
        super(PersonWorkForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            self.fields['work'].initial = instance.related_work.name
            self.fields['work_uri'].initial = Uri.objects.filter(entity=instance.related_work)[0]
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PersonWorkForm'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            'relation_type',
            'start_date_written',
            'end_date_written',
            Accordion(
                AccordionGroup(
                    'Notes and References',
                    'notes',
                    'references',
                    active=False,
                    css_id="persWork_notes_refs")))
        if entity_type == Work:
            self.fields.pop('work')
            self.fields.pop('work_uri')
            if instance and instance.id:
                self.fields['person'].initial = instance.related_person
                self.fields['person_uri'].initial = Uri.objects.filter(entity=instance.related_person)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonWorkReverseAutocomplete',
                                                                      extra_context={
                                                                          'values': [instance.relation_type.pk],
                                                                          'choices': [
                                                                              instance.relation_type.label_reverse]}
                                                                      )
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonWorkReverseAutocomplete')
            self.helper.layout.insert(1, 'person')
            self.helper.layout.insert(2, 'person_uri')
        else:
            self.fields.pop('person')
            self.fields.pop('person_uri')
            if instance and instance.id:
                self.fields['work'].initial = instance.related_work.name
                self.fields['work_uri'].initial = Uri.objects.filter(entity=instance.related_work)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPersonWorkReverseAutocomplete',
                                                                      extra_context={
                                                                          'values': [instance.relation_type.pk],
                                                                          'choices': [
                                                                              instance.relation_type.label]}
                                                                      )
            self.helper.layout.insert(1, 'work')
            self.helper.layout.insert(2, 'work_uri')

class PersonLabelForm(EntityLabelForm):
    pass

##############################################
# Institutions
##############################################


class InstitutionPlaceForm(forms.ModelForm):
    place = forms.CharField(label='Ort', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    institution = forms.CharField(label='Institution', widget=al.TextWidget('InstitutionAutocomplete'))
    institution_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = InstitutionPlace
        fields = ['relation_type', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type')
        }
        widgets = {'relation_type': al.ChoiceWidget('VCInstitutionPlaceAutocomplete')}

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = InstitutionPlace.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(InstitutionPlaceForm, self).save(commit=False)
        if 'place' in cd.keys():
            x.related_institution = site_instance
            place = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
            x.related_place = place
        elif 'institution' in cd.keys():
            x.related_place = site_instance
            institution = Institution.get_or_create_uri(cd['institution_uri'])
            if not institution:
                institution = GenericRDFParser(cd['institution_uri'], 'Institution').get_or_create()
            x.related_institution = institution
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(InstitutionPlaceForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'InstitutionPlaceForm'
        self.helper.form_tag = False
        if entity_type == Place:
            self.fields.pop('place')
            self.fields.pop('place_uri')
            if instance and instance.id:
                self.fields['institution'].initial = instance.related_institution.name
                self.fields['institution_uri'].initial = Uri.objects.filter(entity=instance.related_institution)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCInstitutionPlaceReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCInstitutionPlaceReverseAutocomplete')
            self.helper.layout = Layout(
                    'relation_type',
                    'institution',
                    'institution_uri',
                    'start_date_written',
                    'end_date_written',
                    Accordion(
                        AccordionGroup(
                            'Notes and References',
                            'notes',
                            'references',
                            active=False,
                            css_id="instPlace_notes_refs")))
        else:
            self.fields.pop('institution')
            self.fields.pop('institution_uri')
            if instance and instance.id:
                self.fields['place'].initial = instance.related_place.name
                self.fields['place_uri'].initial = Uri.objects.filter(entity=instance.related_place)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCInstitutionPlaceReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout = Layout(
                'relation_type',
                'place',
                'place_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="instPlace_notes_refs")))


class InstitutionEventForm(forms.ModelForm):
    event = forms.CharField(label='Event', widget=al.TextWidget('EventAutocomplete'))
    event_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    institution = forms.CharField(label='Institution', widget=al.TextWidget('InstitutionAutocomplete'))
    institution_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = InstitutionEvent
        fields = ['relation_type', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type')
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCInstitutionEventAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = InstitutionEvent.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(InstitutionEventForm, self).save(commit=False)
        if 'event' in cd.keys():
            x.related_institution = site_instance
            event = Event.get_or_create_uri(cd['event_uri'].strip())
            if not event:
                event = GenericRDFParser(cd['event_uri'], 'Event').get_or_create()
            x.related_event = event
        elif 'institution' in cd.keys():
            x.related_event = site_instance
            institution = Institution.get_or_create_uri(cd['institution_uri'].strip())
            if not institution:
                institution = GenericRDFParser(cd['institution_uri'], 'Institution').get_or_create()
            x.related_institution = institution
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(InstitutionEventForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'InstitutionEventForm'
        self.helper.form_tag = False
        if entity_type == Event:
            choices = []
            self.fields.pop('event')
            self.fields.pop('event_uri')
            for x in InstitutionEventRelation.objects.all():
                choices.append((x.pk, x.name_reverse))
            self.fields['relation_type'].choices = choices
            if instance and instance.id:
                self.fields['institution'].initial = instance.related_institution.name
                self.fields['institution_uri'].initial = Uri.objects.filter(entity=instance.related_institution)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCInstitutionEventReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCInstitutionEventReverseAutocomplete')
            self.helper.layout = Layout(
                    'relation_type',
                    'institution',
                    'institution_uri',
                    'start_date_written',
                    'end_date_written',
                    Accordion(
                        AccordionGroup(
                            'Notes and References',
                            'notes',
                            'references',
                            active=False,
                            css_id="instEvent_notes_refs")))
        else:
            self.fields.pop('institution')
            self.fields.pop('institution_uri')
            if instance and instance.id:
                self.fields['event'].initial = instance.related_event.name
                self.fields['event_uri'].initial = Uri.objects.filter(entity=instance.related_event)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCInstitutionEventReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout = Layout(
                'relation_type',
                'event',
                'event_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="instEvent_notes_refs")))


class InstitutionWorkForm(forms.ModelForm):
    work = forms.CharField(label='Work', widget=al.TextWidget('WorkAutocomplete'))
    work_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    institution = forms.CharField(label='Institution', widget=al.TextWidget('InstitutionAutocomplete'))
    institution_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = InstitutionWork
        fields = ['relation_type', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type')
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCInstitutionWorkAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = InstitutionWork.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(InstitutionWorkForm, self).save(commit=False)
        if 'work' in cd.keys():
            x.related_institution = site_instance
            work = Work.get_or_create_uri(cd['work_uri'].strip())
            if not work:
                work = GenericRDFParser(cd['work_uri'], 'Work').get_or_create()
            x.related_work = work
        elif 'institution' in cd.keys():
            x.related_work = site_instance
            institution = Institution.get_or_create_uri(cd['institution_uri'].strip())
            if not institution:
                institution = GenericRDFParser(cd['institution_uri'], 'Institution').get_or_create()
            x.related_institution = institution
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(InstitutionWorkForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'InstitutionWorkForm'
        self.helper.form_tag = False
        if entity_type == Work:
            choices = []
            self.fields.pop('work')
            self.fields.pop('work_uri')
            if instance and instance.id:
                self.fields['institution'].initial = instance.related_institution.name
                self.fields['institution_uri'].initial = Uri.objects.filter(entity=instance.related_institution)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCInstitutionWorkReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCInstitutionWorkReverseAutocomplete')
            self.helper.layout = Layout(
                    'relation_type',
                    'institution',
                    'institution_uri',
                    'start_date_written',
                    'end_date_written',
                    Accordion(
                        AccordionGroup(
                            'Notes and References',
                            'notes',
                            'references',
                            active=False,
                            css_id="instWork_notes_refs")))
        else:
            self.fields.pop('institution')
            self.fields.pop('institution_uri')
            if instance and instance.id:
                self.fields['work'].initial = instance.related_work.name
                self.fields['work_uri'].initial = Uri.objects.filter(entity=instance.related_work)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCInstitutionWorkReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout = Layout(
                'relation_type',
                'work',
                'work_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="instWork_notes_refs")))


class InstitutionInstitutionForm(forms.ModelForm):
    institution = forms.CharField(label='Institution', widget=al.TextWidget('InstitutionAutocomplete'))
    institution_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = InstitutionInstitution
        fields = ['relation_type', 'institution', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Art'),
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCInstitutionInstitutionAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = InstitutionInstitution.objects.select_related(
                'related_institutionA__tempentityclass_ptr__uri',
                'related_institutionB__tempentityclass_ptr__uri').get(pk=instance)
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.relation_type = cd['relation_type']
            x.notes = cd['notes']
            x.references = cd['references']
            if site_instance == x.related_institutionB:
                if x.related_institutionA.uri_set.filter(uri=cd['institution_uri']).count() == 0:
                    institutionA = Institution.get_or_create_uri(cd['institution_uri'])
                    x.related_institutionA = institutionA
        else:
            x = super(InstitutionInstitutionForm, self).save(commit=False)
            x.related_institutionA = site_instance
            institutionB = Institution.get_or_create_uri(cd['institution_uri'])
            if not institutionB:
                institutionB = GenericRDFParser(['institution_uri'], 'Institution').get_or_create()
            x.related_institutionB = institutionB
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(InstitutionInstitutionForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            if siteID:
                siteID = Institution.objects.get(pk=siteID)
                if siteID == instance.related_institutionB:
                    rel_inst = instance.related_institutionA
                    self.fields['relation_type'].widget = al.ChoiceWidget(
                        'VCInstitutionInstitutionReverseAutocomplete',
                        extra_context={'values': [instance.relation_type.pk],
                                       'choices': [instance.relation_type.label_reverse]})
                else:
                    rel_inst = instance.related_institutionB
            self.fields['institution'].initial = rel_inst.name
            try:
                self.fields['institution_uri'].initial = Uri.objects.filter(
                    entity=rel_inst)[0]
            except:
                pass
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'InstitutionInstitutionForm'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            'relation_type',
            'institution',
            'institution_uri',
            'start_date_written',
            'end_date_written',
            Accordion(
                AccordionGroup(
                    'Notes and References',
                    'notes',
                    'references',
                    active=False,
                    css_id="instInst_notes_refs")))


class InstitutionPersonForm(forms.ModelForm):
    person = forms.CharField(
        label='Person',
        widget=al.TextWidget('PersonAutocomplete'),
        )
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = PersonInstitution
        fields = ['relation_type', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type'),
            'related_person': _('Person')
        }
        widgets = {
            'related_person': al.ChoiceWidget('PersonAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PersonInstitution.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(InstitutionPersonForm, self).save(commit=False)
            x.related_institution = site_instance
        person = Person.get_or_create_uri(cd['person_uri'])
        if not person:
            person = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()
        x.related_person = person
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        self.request = kwargs.pop('request', False)
        super(InstitutionPersonForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            self.fields['person'].initial = instance.related_person.name+", "+instance.related_person.first_name
            self.fields['person_uri'].initial = Uri.objects.filter(entity=instance.related_person)[0]
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'InstitutionPersonForm'
        self.helper.form_tag = False


class InstitutionLabelForm(EntityLabelForm):
    pass


##############################################
# Places
##############################################


class PlaceEventForm(forms.ModelForm):
    event = forms.CharField(label='Event', widget=al.TextWidget('EventAutocomplete'))
    event_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    place = forms.CharField(label='Place', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = PlaceEvent
        fields = ['relation_type', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type')
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCPlaceEventAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PlaceEvent.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(PlaceEventForm, self).save(commit=False)
        if 'event' in cd.keys():
            x.related_place = site_instance
            event = Event.get_or_create_uri(cd['event_uri'])
            if not event:
                event = GenericRDFParser(cd['event_uri'], 'Event').get_or_create()
            x.related_event = event
        elif 'place' in cd.keys():
            x.related_event = site_instance
            place = Place.get_or_create_uri(cd['place_uri'])
            if not place:
                place = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
            x.related_place = place
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(PlaceEventForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PlaceEventForm'
        self.helper.form_tag = False
        if entity_type == Event:
            self.fields.pop('event')
            self.fields.pop('event_uri')

            if instance and instance.id:
                self.fields['place'].initial = instance.related_place.name
                self.fields['place_uri'].initial = Uri.objects.filter(entity=instance.related_place)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPlaceEventReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPlaceEventReverseAutocomplete')
            self.helper.layout = Layout(
                    'relation_type',
                    'place',
                    'place_uri',
                    'start_date_written',
                    'end_date_written',
                    Accordion(
                        AccordionGroup(
                            'Notes and References',
                            'notes',
                            'references',
                            active=False,
                            css_id="placeEvent_notes_refs")))
        else:
            self.fields.pop('place')
            self.fields.pop('place_uri')
            if instance and instance.id:
                self.fields['event'].initial = instance.related_event.name
                self.fields['event_uri'].initial = Uri.objects.filter(entity=instance.related_event)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPlaceEventReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout = Layout(
                'relation_type',
                'event',
                'event_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="placeEvent_notes_refs")))


class PlaceWorkForm(forms.ModelForm):
    work = forms.CharField(label='Work', widget=al.TextWidget('WorkAutocomplete'))
    work_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    place = forms.CharField(label='Place', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = PlaceWork
        fields = ['relation_type', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type')
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCPlaceWorkAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PlaceWork.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(PlaceWorkForm, self).save(commit=False)
        if 'work' in cd.keys():
            x.related_place = site_instance
            work = Work.get_or_create_uri(cd['work_uri'])
            if not work:
                work = GenericRDFParser(cd['work_uri'], 'Work').get_or_create()
            x.related_work = work
        elif 'place' in cd.keys():
            x.related_work = site_instance
            place = Place.get_or_create_uri(cd['place_uri'])
            if not place:
                place = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
            x.related_place = place
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(PlaceWorkForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PlaceWorkForm'
        self.helper.form_tag = False
        if entity_type == Work:
            self.fields.pop('work')
            self.fields.pop('work_uri')

            if instance and instance.id:
                self.fields['place'].initial = instance.related_place.name
                self.fields['place_uri'].initial = Uri.objects.filter(entity=instance.related_place)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPlaceWorkReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCPlaceWorkReverseAutocomplete')
            self.helper.layout = Layout(
                'relation_type',
                'place',
                'place_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="placeWork_notes_refs")))
        else:
            self.fields.pop('place')
            self.fields.pop('place_uri')
            if instance and instance.id:
                self.fields['work'].initial = instance.related_work.name
                self.fields['work_uri'].initial = Uri.objects.filter(entity=instance.related_work)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCPlaceWorkReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout = Layout(
                'relation_type',
                'work',
                'work_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="placeWork_notes_refs")))


class PlacePlaceForm(forms.ModelForm):
    place = forms.CharField(label='Place', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = PlacePlace
        fields = ['relation_type', 'place', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Art'),
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCPlacePlaceAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = PlacePlace.objects.select_related(
                'related_placeA__tempentityclass_ptr__uri',
                'related_placeB__tempentityclass_ptr__uri').get(pk=instance)
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.relation_type = cd['relation_type']
            x.notes = cd['notes']
            x.references = cd['references']
            if site_instance == x.related_placeB:
                if x.related_placeA.uri_set.filter(uri=cd['place_uri']).count() == 0:
                    placeA = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
                    x.related_placeA = placeA
        else:
            x = super(PlacePlaceForm, self).save(commit=False)
            x.related_placeA = site_instance
            placeB = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
            x.related_placeB = placeB
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        if not hasattr(self, 'request'):
            self.request = kwargs.pop('request', False)
        super(PlacePlaceForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            if siteID:
                siteID = Place.objects.get(pk=siteID)
                if siteID == instance.related_placeB:
                    rel_place = instance.related_placeA
                    self.fields['relation_type'].widget = al.ChoiceWidget(
                        'VCPlacePlaceReverseAutocomplete',
                        extra_context={'values': [instance.relation_type.pk],
                                       'choices': [instance.relation_type.label_reverse]})
                else:
                    rel_place = instance.related_placeB
            self.fields['place'].initial = rel_place
            self.fields['place_uri'].initial = Uri.objects.filter(entity=rel_place)[0]
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'PlacePlaceForm'
        self.helper.form_tag = False
        self.helper.layout = Layout(
            'relation_type',
            'place',
            'place_uri',
            'start_date_written',
            'end_date_written',
            Accordion(
                AccordionGroup(
                    'Notes and References',
                    'notes',
                    'references',
                    active=False,
                    css_id="placePlace_notes_refs")))


##############################################
# Events
##############################################


class EventWorkForm(forms.ModelForm):
    work = forms.CharField(label='Work', widget=al.TextWidget('WorkAutocomplete'))
    work_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    event = forms.CharField(label='Event', widget=al.TextWidget('EventAutocomplete'))
    event_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    class Meta:
        model = EventWork
        fields = ['relation_type', 'start_date_written', 'end_date_written', 'references', 'notes']
        labels = {
            'start_date_written': _('Start'),
            'end_date_written': _('End'),
            'relation_type': _('Type')
        }
        widgets = {
            'relation_type': al.ChoiceWidget('VCEventWorkAutocomplete')
        }

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        if instance:
            x = EventWork.objects.get(pk=instance)
            x.relation_type = cd['relation_type']
            x.start_date_written = cd['start_date_written']
            x.end_date_written = cd['end_date_written']
            x.notes = cd['notes']
            x.references = cd['references']
        else:
            x = super(EventWorkForm, self).save(commit=False)
        if 'work' in cd.keys():
            x.related_event = site_instance
            work = Work.get_or_create_uri(cd['work_uri'])
            if not work:
                work = GenericRDFParser(cd['work_uri'], 'Work').get_or_create()
            x.related_work = work
        elif 'event' in cd.keys():
            x.related_work = site_instance
            event = Event.get_or_create_uri(cd['event_uri'])
            if not event:
                event = GenericRDFParser(cd['event_uri'], 'Event').get_or_create()
            x.related_event = event
        x.save()
        return x

    def __init__(self, siteID=None, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(EventWorkForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        self.fields['relation_type'].required = True
        self.helper = FormHelper()
        self.helper.form_class = 'EventWorkForm'
        self.helper.form_tag = False
        if entity_type == Work:
            self.fields.pop('work')
            self.fields.pop('work_uri')

            if instance and instance.id:
                self.fields['event'].initial = instance.related_event.name
                self.fields['event_uri'].initial = Uri.objects.filter(entity=instance.related_event)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCEventWorkReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label_reverse]})
            else:
                self.fields['relation_type'].widget = al.ChoiceWidget('VCEventWorkReverseAutocomplete')
            self.helper.layout = Layout(
                'relation_type',
                'event',
                'event_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="eventWork_notes_refs")))
        else:
            self.fields.pop('event')
            self.fields.pop('event_uri')
            if instance and instance.id:
                self.fields['work'].initial = instance.related_work.name
                self.fields['work_uri'].initial = Uri.objects.filter(entity=instance.related_work)[0]
                self.fields['relation_type'].widget = al.ChoiceWidget(
                    'VCEventWorkReverseAutocomplete',
                    extra_context={'values': [instance.relation_type.pk],
                                   'choices': [instance.relation_type.label]})
            self.helper.layout = Layout(
                'relation_type',
                'work',
                'work_uri',
                'start_date_written',
                'end_date_written',
                Accordion(
                    AccordionGroup(
                        'Notes and References',
                        'notes',
                        'references',
                        active=False,
                        css_id="eventWork_notes_refs")))


class EventLabelForm(EntityLabelForm):
    pass


##############################################
# Entities Base Forms
#############################################

class PlaceEntityForm(forms.Form):
    place = forms.CharField(label='Place', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        pl = Place.get_or_create_uri(cd['place_uri'])
        if not pl:
            pl = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
        return pl

##############################################
# Highlighter
##############################################


class PlaceEntityHighlighterForm(forms.Form):
    place = forms.CharField(label='Place', widget=al.TextWidget('OrtAutocomplete'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        pl = Place.get_or_create_uri(cd['place_uri'])
        if not pl:
            pl = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(pl)
        return a

    def __init__(self, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(PlaceEntityHighlighterForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = 'PlaceEntityForm'
        self.helper.form_tag = False


class PersonPlaceHighlighterForm(PersonPlaceForm):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        #self.request = kwargs.pop('request', False)
        super(PersonPlaceHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.layout.pop(-1)
        self.helper.layout.extend([
            'HL_start',
            'HL_end',
            'HL_text_id',
            Accordion(
                 AccordionGroup(
                     'Notes and References',
                     'notes',
                     'references',
                     active=False,
                     css_id="persPlaceHL_notes_refs"))])

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        x = super(PersonPlaceHighlighterForm, self).save(site_instance, commit=True)
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(x)
        return x

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]


class PersonInstitutionHighlighterForm(PersonInstitutionForm):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        #self.request = kwargs.pop('request', False)
        super(PersonInstitutionHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.layout.pop(-1)
        self.helper.layout.extend([
            'HL_start',
            'HL_end',
            'HL_text_id',
            Accordion(
                 AccordionGroup(
                     'Notes and References',
                     'notes',
                     'references',
                     active=False,
                     css_id="persInstHL_notes_refs"))])

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        x = super(PersonInstitutionHighlighterForm, self).save(site_instance, commit=True)
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(x)
        return x

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]


class PersonPersonHighlighterForm(PersonPersonForm):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        #self.request = kwargs.pop('request', False)
        super(PersonPersonHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.layout.pop(-1)
        self.helper.layout.extend([
            'HL_start',
            'HL_end',
            'HL_text_id',
            Accordion(
                 AccordionGroup(
                     'Notes and References',
                     'notes',
                     'references',
                     active=False,
                     css_id="persPlaceHL_notes_refs"))])

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        x = super(PersonPersonHighlighterForm, self).save(site_instance, commit=True)
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(x)
        return x

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]


class PlaceWorkHighlighterForm(PlaceWorkForm):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        #self.request = kwargs.pop('request', False)
        super(PlaceWorkHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.layout.pop(-1)
        self.helper.layout.extend([
            'HL_start',
            'HL_end',
            'HL_text_id',
            Accordion(
                 AccordionGroup(
                     'Notes and References',
                     'notes',
                     'references',
                     active=False,
                     css_id="placeWorkHL_notes_refs"))])

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        x = super(PlaceWorkHighlighterForm, self).save(site_instance, commit=True)
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(x)
        return x

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]


class PersonWorkHighlighterForm(PersonWorkForm):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        #self.request = kwargs.pop('request', False)
        super(PersonWorkHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.layout.pop(-1)
        self.helper.layout.extend([
            'HL_start',
            'HL_end',
            'HL_text_id',
            Accordion(
                 AccordionGroup(
                     'Notes and References',
                     'notes',
                     'references',
                     active=False,
                     css_id="personWorkHL_notes_refs"))])

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        x = super(PersonWorkHighlighterForm, self).save(site_instance, commit=True)
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(x)
        return x

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]


class InstitutionWorkHighlighterForm(InstitutionWorkForm):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        #self.request = kwargs.pop('request', False)
        super(InstitutionWorkHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.layout.pop(-1)
        self.helper.layout.extend([
            'HL_start',
            'HL_end',
            'HL_text_id',
            Accordion(
                 AccordionGroup(
                     'Notes and References',
                     'notes',
                     'references',
                     active=False,
                     css_id="institutionWorkHL_notes_refs"))])

    def save(self, site_instance, instance=None, commit=True):
        cd = self.cleaned_data
        x = super(InstitutionWorkHighlighterForm, self).save(site_instance, commit=True)
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(x)
        return x

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]


class AddRelationHighlighterBaseForm(forms.Form):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)
    RL_type = forms.CharField(widget=forms.HiddenInput)
    RL_pk = forms.IntegerField(widget=forms.HiddenInput)

    def save(self):
        cd = self.cleaned_data
        #x = super(AddRelationHighlighterBaseForm, self).save()
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        ent = ContentType.objects.get(
            pk=cd['RL_type']).model_class().objects.get(pk=cd['RL_pk'])
        self.ann = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user)
        self.ann.save()
        self.ann.entity_link.add(ent)
        return self.ann


class AddRelationHighlighterPersonForm(AddRelationHighlighterBaseForm):
    relation = forms.CharField(
        label='Relation',
        widget=al.TextWidget('AddRelationPersonHighlighterAutocomplete'),
        )

    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        x = super().save()

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', False)
        self.entity_type = kwargs.pop('entity_type', False)
        super(AddRelationHighlighterPersonForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]




from django.test import TestCase
from django.contrib.auth.models import User, Group

from .models import Person, Place, Institution, Work, Event
from apis_core.apis_metainfo.models import Text, Collection, Source, Uri, UriCandidate
from apis_core.apis_vocabularies.models import PersonPlaceRelation
from apis_core.apis_labels.models import Label
from apis_core.apis_relations.models import PersonPlace
from reversion.models import Version
from reversion import revisions as reversion
from apis_core.helper_functions.RDFparsers import GenericRDFParser

from datetime import datetime
from guardian.shortcuts import assign_perm, remove_perm, get_objects_for_user
from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient
from django.contrib.auth.models import Permission


class PersonModelTestCase(TestCase):
    name = 'test name'
    first_name = 'test first name'
    text = 'kjhkjsdhaslkdhadh lasdjalsk alsjdhaksjdh'
    col_name = 'Test collection'
    start_date = '1.3.1930'
    end_date = '4.6.1960'

    @classmethod
    def setUpTestData(cls):
        # Set up data for the whole TestCase
        cls.txt = Text.objects.create(text=cls.text)
        cls.col = Collection.objects.create(name=cls.col_name)

    def test_init(self):
        Person()

    def test_create_object(self):
        p = Person.objects.create(
            name=self.name,
            first_name=self.first_name,
            start_date_written=self.start_date,
            end_date_written=self.end_date)

    def test_object_connections(self):
        p = Person.objects.create(
            name=self.name,
            first_name=self.first_name,
            start_date_written=self.start_date,
            end_date_written=self.end_date)
        p.collection.add(self.col)
        p.text.add(self.txt)

    def test_object_validity(self):
        p = Person.objects.create(
            name=self.name,
            first_name=self.first_name,
            start_date_written=self.start_date,
            end_date_written=self.end_date)
        p.collection.add(self.col)
        p.text.add(self.txt)
        self.assertEqual(p.name, self.name)
        self.assertEqual(p.first_name, self.first_name)
        self.assertEqual(p.start_date_written, self.start_date)
        self.assertEqual(p.start_date, datetime.strptime(self.start_date, '%d.%m.%Y'))

    def test_object_reversion(self):
        with reversion.create_revision():
            p = Person.objects.create(
                name=self.name,
                first_name=self.first_name,
                start_date_written=self.start_date,
                end_date_written=self.end_date)
            p.collection.add(self.col)
            p.text.add(self.txt)
        with reversion.create_revision():
            p2 = Person.objects.get(name=self.name)
            p2.name = 'testname 2'
            p2.save()
        self.assertEqual(p2.name, 'testname 2')
        versions = Version.objects.get_for_object(p2)
        self.assertEqual(len(versions), 2)
        self.assertEqual(versions[0].field_dict['name'], 'testname 2')
        self.assertEqual(versions[1].field_dict['name'], self.name)


class PermissionsModelTestCase(TestCase):
    name = 'test name'
    first_name = 'test first name'
    text = 'kjhkjsdhaslkdhadh lasdjalsk alsjdhaksjdh'
    col_name = 'Test collection'
    start_date = '1.3.1930'
    end_date = '4.6.1960'

    @classmethod
    def setUpTestData(cls):
        # Set up data for the whole TestCase
        cls.txt = Text.objects.create(text=cls.text)
        cls.col = Collection.objects.create(name=cls.col_name)
        cls.pers = Person.objects.create(
            name=cls.name,
            first_name=cls.first_name,
            start_date_written=cls.start_date,
            end_date_written=cls.end_date)
        cls.user = User.objects.create_user(
            'testuser',
            'apisdev16')
        pe = Permission.objects.get(name="Can change person")
        cls.user.user_permissions.add(pe)
        cls.token = Token.objects.create(user=cls.user).key
        cls.group = Group.objects.create(name='testgroup')
        cls.col.groups_allowed.add(cls.group)
        cls.user.groups.add(cls.group)
        cls.c = APIClient()
        cls.c.credentials(HTTP_AUTHORIZATION='Token ' + cls.token)

    def test_no_perm(self):
        res = self.c.patch(
            '/api/person/'+str(self.pers.pk)+'/',
            data={
            'name': 'changed name'
            },
            #format='json'
            )
        print(res.content)
        self.assertEqual(res.status_code, 403)

    def test_perm(self):
        self.pers.collection.add(self.col)
        res = self.c.patch(
            '/api/person/'+str(self.pers.pk)+'/',
            data={
            'name': 'changed name'
            },
            #format='json'
            )
        self.assertEqual(res.status_code, 200)

    def test_perm_removed(self):
        self.pers.collection.remove(self.col)
        res = self.c.patch(
            '/api/person/'+str(self.pers.pk)+'/',
            data={
            'name': 'changed name'
            },
            #format='json'
            )
        self.assertEqual(res.status_code, 403)





import django_tables2 as tables
from django.contrib.contenttypes.models import ContentType
from django_tables2.utils import A

from .models import (
    PersonInstitution, PersonPlace, PersonPerson, PersonEvent, InstitutionEvent, PlaceEvent,
    PersonWork, InstitutionWork, InstitutionPlace, PlaceWork, EventWork
)
from apis_core.apis_labels.models import Label
from apis_core.apis_metainfo.models import Uri


empty_text_default = 'There are currently no relations'


def get_generic_relation_listview_table(relation):
    ct = ContentType.objects.get(app_label='apis_relations', model=relation.lower())
    relation_model_class = ct.model_class()
    model_fields = relation_model_class._meta.get_fields(include_parents=False)
    field_names = [x.name for x in model_fields[1:]]
    all_field_names = field_names + ['start_date', 'end_date']

    class GenericRelationListViewTable(tables.Table):

        class Meta:
            model = relation_model_class
            fields = all_field_names
            attrs = {"class": "table table-hover table-striped table-condensed"}

        def __init__(self, *args, **kwargs):
            ent_first = all_field_names[1]
            ent_second = all_field_names[2]
            self.base_columns[all_field_names[1]] = tables.LinkColumn()
            self.base_columns[all_field_names[2]] = tables.LinkColumn()
            super(GenericRelationListViewTable, self).__init__(*args, **kwargs)

    return GenericRelationListViewTable


def get_generic_relations_table(relation, entity, detail=None):
    ct = ContentType.objects.get(app_label='apis_relations', model=relation.lower())
    name = ct.name.split()
    if name[0] == name[1]:
        rel_ent = name[0]
    else:
        if name[1].lower() == entity.lower():
            rel_ent = name[0]
            rel_type = 'label_reverse'
        else:
            rel_ent = name[1]
            rel_type = 'label'

    class GenericRelationsTable(tables.Table):

        class Meta:
            empty_text = empty_text_default
            model = ct.model_class()
            fields = ['start_date', 'end_date']
            attrs = {
                "class": "table table-hover table-striped table-condensed",
                "id": name[0].title()[:2]+name[1].title()[:2]+"_conn"}

        def __init__(self, *args, entity='None', **kwargs):
            self.base_columns['delete'] = tables.TemplateColumn(
                template_name='apis_relations/delete_button_generic_ajax_form.html'
            )
            self.base_columns['related_'+rel_ent.lower()] = tables.LinkColumn(
                'apis:apis_entities:generic_entities_edit_view',
                args=[
                    rel_ent.lower(), A('related_{}.pk'.format(rel_ent.lower()))
                ])
            if name[0] == name[1]:
                self.base_columns['relation_type'] = tables.Column()
            else:
                self.base_columns['relation_type'] = tables.Column(
                    verbose_name='relation type', accessor='relation_type.{}'.format(rel_type))
            self.base_columns['edit'] = tables.TemplateColumn(
                template_name='apis_relations/edit_button_generic_ajax_form.html'
            )
            super(GenericRelationsTable, self).__init__(*args, **kwargs)
            self.sequence = (
                'delete', 'start_date', 'end_date', 'relation_type',
                'related_' + rel_ent.lower(), 'edit'
            )

    if detail:

        class GenericRelationsDetailTable(GenericRelationsTable):

            class Meta:
                empty_text = empty_text_default
                model = ct.model_class()
                fields = ['start_date', 'end_date']
                attrs = {
                    "class": "table table-hover table-striped table-condensed",
                    "id": name[0].title()[:2]+name[1].title()[:2]+"_conn"}

            def __init__(self, *args, entity='None', **kwargs):
                self.base_columns['related_'+rel_ent.lower()] = tables.LinkColumn(
                    'apis:apis_entities:generic_entities_detail_view',
                    args=[
                        rel_ent.lower(), A('related_{}.pk'.format(rel_ent.lower()))
                    ])
                if name[0] == name[1]:
                    self.base_columns['relation_type'] = tables.Column()
                else:
                    self.base_columns['relation_type'] = tables.Column(
                        verbose_name='relation type', accessor='relation_type.{}'.format(rel_type))
                super(GenericRelationsTable, self).__init__(*args, **kwargs)
                self.sequence = (
                    'start_date', 'end_date', 'relation_type', 'related_' + rel_ent.lower()
                )
        return GenericRelationsDetailTable
    else:
        return GenericRelationsTable


class EntityUriTable(tables.Table):
    delete = tables.TemplateColumn(template_name='apis_relations/delete_button_Uri_ajax_form.html')

    class Meta:
        empty_text = empty_text_default
        model = Uri
        fields = ['uri']
        sequence = ('delete', 'uri')
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed",
                 "id": "PURI_conn"}


class EntityDetailViewLabelTable(tables.Table):
    label2 = tables.Column(accessor='label')

    class Meta:
        empty_text = empty_text_default
        model = Label
        fields = ['isoCode_639_3', 'label_type']
        sequence = ('label2', 'label_type', 'isoCode_639_3')
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed",
                "id": "PL_conn"}



class EntityLabelTable(tables.Table):
    edit = tables.TemplateColumn(template_name='apis_relations/edit_button_persLabel_ajax_form.html')
    label2 = tables.Column(accessor='label')

    class Meta:
        empty_text = empty_text_default
        model = Label
        fields = ['isoCode_639_3', 'label_type']
        sequence = ('label2', 'label_type', 'isoCode_639_3')
        # add class="paleblue" to <table> tag
        attrs = {"class": "table table-hover table-striped table-condensed",
                "id": "PL_conn"}



# Generated by Django 2.1.2 on 2018-10-03 13:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_entities', '0001_initial'),
        ('apis_vocabularies', '0001_initial'),
        ('apis_metainfo', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventEvent',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_eventA', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_eventA', to='apis_entities.Event')),
                ('related_eventB', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_eventB', to='apis_entities.Event')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.EventEventRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='EventWork',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_event', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Event')),
                ('related_work', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Work')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.EventWorkRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='InstitutionEvent',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_event', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Event')),
                ('related_institution', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Institution')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.InstitutionEventRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='InstitutionInstitution',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_institutionA', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_institutionA', to='apis_entities.Institution')),
                ('related_institutionB', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_institutionB', to='apis_entities.Institution')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.InstitutionInstitutionRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='InstitutionPlace',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_institution', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Institution')),
                ('related_place', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Place')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.InstitutionPlaceRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='InstitutionWork',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_institution', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Institution')),
                ('related_work', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Work')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.InstitutionWorkRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PersonEvent',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_event', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Event')),
                ('related_person', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Person')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PersonEventRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PersonInstitution',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_institution', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Institution')),
                ('related_person', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Person')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PersonInstitutionRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PersonPerson',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_personA', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_personA', to='apis_entities.Person')),
                ('related_personB', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_personB', to='apis_entities.Person')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PersonPersonRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PersonPlace',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_person', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Person')),
                ('related_place', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Place')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PersonPlaceRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PersonWork',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_person', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Person')),
                ('related_work', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Work')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PersonWorkRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PlaceEvent',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_event', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Event')),
                ('related_place', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Place')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PlaceEventRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PlacePlace',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_placeA', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_place_placeA', to='apis_entities.Place')),
                ('related_placeB', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_place_placeB', to='apis_entities.Place')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PlacePlaceRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='PlaceWork',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_place', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Place')),
                ('related_work', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='apis_entities.Work')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.PlaceWorkRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
        migrations.CreateModel(
            name='WorkWork',
            fields=[
                ('tempentityclass_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_metainfo.TempEntityClass')),
                ('related_workA', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_workA', to='apis_entities.Work')),
                ('related_workB', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_workB', to='apis_entities.Work')),
                ('relation_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.WorkWorkRelation')),
            ],
            bases=('apis_metainfo.tempentityclass',),
        ),
    ]



import os
import re

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.rst")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


def get_version(*file_paths):
    """Retrieves the version from apis_core/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


version = get_version("apis_core", "__init__.py")
bower_json = {
    "dependencies": {
        "jquery": "^3.2.1",
        "bootstrap": "^3.3.7",
        "bootstrap-multiselect": "^0.9.13",
        "bootstrap-datepicker": "^1.7.0",
        "jquery-tablesort": "^0.0.11",
        "leaflet": "^1.1.0",
        "tooltipster": "^4.2.5",
        "leaflet.markercluster": "Leaflet.markercluster#^1.0.6",
    },
    "resolutions": {"jquery": "1.9.1 - 3"},
}

setup(
    name="apis-core",
    version=version,
    packages=find_packages(),
    include_package_data=True,
    license="MIT License",  # example license
    description="APIS core package. Includes entities, relations, vocabularies,\
    labels and the helper scripts for RDF parsing.",
    long_description=README,
    url="https://www.apis.acdh.oeaw.ac.at/",
    author="Matthias Schlögl, Peter Andorfer",
    author_email="matthias.schloegl@oeaw.ac.at, peter.andorfer@oeaw.ac.at",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.0",  # replace "X.Y" as appropriate
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",  # example license
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    install_requires=[
        "calmjs.bower",
        "Django>=2.0",
        "django-autocomplete-light>=3.2.10",
        "django-crispy-forms>=1.7.0",
        "django-filter>=1.1.0",
        "django-gm2m>=0.6.1",
        "django-guardian>=1.4.9",
        "django-model-utils>=3.1.2",
        "django-reversion>=2.0.13",
        "django-reversion-compare>=0.8.4",
        "django-tables2>=1.21.1",
        "djangorestframework>=3.7.7",
        "djangorestframework-csv>=2.1.0",
        "djangorestframework-xml>=1.3.0",
        "lxml>=4.2.4",
        "python-dateutil>=2.7.0",
        "rdflib>=4.2.2",
        "requests>=2.18.4",
    ],
    bower_json=bower_json,
)



import re
import unicodedata
from datetime import datetime
from difflib import SequenceMatcher

import requests

# from reversion import revisions as reversion
import reversion
from apis_core.apis_entities.serializers_generic import EntitySerializer
from apis_core.apis_labels.models import Label
from apis_core.apis_vocabularies.models import CollectionType, LabelType, TextType

# from helper_functions.highlighter import highlight_text
from apis_core.default_settings.NER_settings import autocomp_settings
from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.query import QuerySet
from django.db.models.signals import m2m_changed, post_save
from django.dispatch import receiver
from django.urls import NoReverseMatch, reverse
from django.utils.functional import cached_property
from model_utils.managers import InheritanceManager

from .validators import date_validator

NEXT_PREV = getattr(settings, "APIS_NEXT_PREV", True)


if "apis_highlighter" in settings.INSTALLED_APPS:
    from apis_highlighter.models import Annotation


@reversion.register()
class TempEntityClass(models.Model):
    """ Base class to bind common attributes to many classes.

    The common attributes are:
    written start and enddates
    recognized start and enddates which are derived by RegEx
    from the written dates.
    A review boolean field to mark an object as reviewed
    """

    name = models.CharField(max_length=255, blank=True)
    review = models.BooleanField(
        default=False,
        help_text="Should be set to True, if the data record holds up quality standards.",
    )
    start_date = models.DateField(blank=True, null=True)
    end_date = models.DateField(blank=True, null=True)
    start_date_written = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        validators=[date_validator],
        verbose_name="Start",
        help_text="Please enter a date (DD).(MM).YYYY",
    )
    end_date_written = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        validators=[date_validator],
        verbose_name="End",
        help_text="Please enter a date (DD).(MM).YYYY",
    )
    text = models.ManyToManyField("Text", blank=True)
    collection = models.ManyToManyField("Collection")
    status = models.CharField(max_length=100)
    source = models.ForeignKey(
        "Source", blank=True, null=True, on_delete=models.SET_NULL
    )
    references = models.TextField(blank=True, null=True)
    notes = models.TextField(blank=True, null=True)
    objects = models.Manager()
    objects_inheritance = InheritanceManager()

    def __str__(self):
        if self.name != "" and hasattr(
            self, "first_name"
        ):  # relation usually don´t have names
            return "{}, {}".format(self.name, self.first_name)
        elif self.name != "":
            return self.name
        else:
            return "(ID: {})".format(self.id)

    def save(self, parse_dates=True, *args, **kwargs):
        """Adaption of the save() method of the class to automatically parse string-dates into date objects
        """

        def match_date(date):
            """Function to parse string-dates into python date objects.
            """
            date = date.strip()
            date = date.replace("-", ".")
            if re.match(r"[0-9]{4}$", date):
                dr = datetime.strptime(date, "%Y")
                dr2 = date
            elif re.match(r"[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{4}$", date):
                dr = datetime.strptime(date, "%d.%m.%Y")
                dr2 = date
            elif re.match(r"[0-9]{4}\.\.\.$", date):
                dr = datetime.strptime(date, "%Y...")
                dr2 = re.match(r"([0-9]{4})\.\.\.$", date).group(1)
            elif re.match(r"[0-9]{4}\.[0-9]{1,2}\.\.$", date):
                dr = datetime.strptime(date, "%Y.%m..")
                dr2 = re.match(r"([0-9]{4})\.([0-9]{1,2})\.\.$", date).group(2)
                +"." + re.match(r"([0-9]{4})\.([0-9]{1,2})\.\.$", date).group(1)
            elif re.match(r"[0-9]{4}\.[0-9]{1,2}\.[0-9]{1,2}$", date):
                dr = datetime.strptime(date, "%Y.%m.%d")
                dr3 = re.match(r"([0-9]{4})\.([0-9]{1,2})\.([0-9]{1,2})$", date)
                dr2 = dr3.group(3) + "." + dr3.group(2) + "." + dr3.group(1)
            elif re.match(r"^\s*$", date):
                dr = None
                dr2 = None
            else:
                dr = None
                dr2 = date
            return dr, dr2

        if parse_dates:
            if self.start_date_written:
                self.start_date, self.start_date_written = match_date(
                    self.start_date_written
                )
            else:
                self.start_date = self.start_date_written = None
            if self.end_date_written:
                self.end_date, self.end_date_written = match_date(self.end_date_written)
            else:
                self.end_date = self.end_date_written = None
            if self.name:
                self.name = unicodedata.normalize("NFC", self.name)
            super(TempEntityClass, self).save(*args, **kwargs)
        else:
            if self.name:
                self.name = unicodedata.normalize("NFC", self.name)
            super(TempEntityClass, self).save(*args, **kwargs)
        return self

    @classmethod
    def get_listview_url(self):
        entity = self.__name__.lower()
        if entity == "institution" or len(entity) < 10:
            return reverse(
                "apis_core:apis_entities:generic_entities_list",
                kwargs={"entity": entity},
            )
        else:
            return reverse(
                "apis_core:apis_relations:generic_relations_list",
                kwargs={"entity": entity},
            )

    @classmethod
    def get_createview_url(self):
        entity = self.__name__.lower()
        if entity == "institution" or len(entity) < 10:
            return reverse(
                "apis_core:apis_entities:generic_entities_create_view",
                kwargs={"entity": entity},
            )
        else:
            return None

    def get_edit_url(self):
        entity = self.__class__.__name__.lower()
        if entity == "institution" or len(entity) < 10:
            return reverse(
                "apis_core:apis_entities:generic_entities_edit_view",
                kwargs={"entity": entity, "pk": self.id},
            )
        else:
            return None

    def get_absolute_url(self):
        entity = self.__class__.__name__.lower()
        if entity == "institution" or len(entity) < 10:
            return reverse(
                "apis_core:apis_entities:generic_entities_detail_view",
                kwargs={"entity": entity, "pk": self.id},
            )
        else:
            return reverse(
                "apis_core:apis_relations:generic_relations_detail_view",
                kwargs={"entity": entity, "pk": self.id},
            )

    def get_prev_url(self):
        entity = self.__class__.__name__.lower()
        if NEXT_PREV:
            prev = self.__class__.objects.filter(id__lt=self.id).order_by("-id")
        else:
            return False
        if prev:
            if entity == "institution" or len(entity) < 10:
                return reverse(
                    "apis_core:apis_entities:generic_entities_detail_view",
                    kwargs={"entity": entity, "pk": prev.first().id},
                )
            else:
                return reverse(
                    "apis_core:apis_relations:generic_relations_detail_view",
                    kwargs={"entity": entity, "pk": prev.first().id},
                )
        else:
            return False

    def get_next_url(self):
        entity = self.__class__.__name__.lower()
        if NEXT_PREV:
            next = self.__class__.objects.filter(id__gt=self.id)
        else:
            return False
        if next:
            if entity == "institution" or len(entity) < 10:
                return reverse(
                    "apis_core:apis_entities:generic_entities_detail_view",
                    kwargs={"entity": entity, "pk": next.first().id},
                )
            else:
                return reverse(
                    "apis_core:apis_relations:generic_relations_detail_view",
                    kwargs={"entity": entity, "pk": next.first().id},
                )
        else:
            return False

    def get_delete_url(self):
        entity = self.__class__.__name__.lower()
        if entity == "institution" or len(entity) < 10:
            return reverse(
                "apis_core:apis_entities:generic_entities_delete_view",
                kwargs={"entity": entity, "pk": self.id},
            )
        else:
            return None

    def merge_with(self, entities):
        e_a = type(self).__name__
        self_model_class = ContentType.objects.get(
            app_label="apis_entities", model__iexact=e_a
        ).model_class()
        if isinstance(entities, int):
            entities = self_model_class.objects.get(pk=entities)
        if not isinstance(entities, list) and not isinstance(entities, QuerySet):
            entities = [entities]
            entities = [
                self_model_class.objects.get(pk=ent) if type(ent) == int else ent
                for ent in entities
            ]
        rels = ContentType.objects.filter(
            app_label="apis_relations", model__icontains=e_a
        )
        for ent in entities:
            e_b = type(ent).__name__
            if e_a != e_b:
                continue
            print(e_b)
            print(str(ent))
            lt, created = LabelType.objects.get_or_create(name="Legacy name (merge)")
            l_uri, created = LabelType.objects.get_or_create(name="Legacy URI (merge)")
            Label.objects.create(label=str(ent), label_type=lt, temp_entity=self)
            for u in Uri.objects.filter(entity=ent):
                Label.objects.create(
                    label=str(u.uri), label_type=l_uri, temp_entity=self
                )
            for l in Label.objects.filter(temp_entity=ent):
                l.temp_entity = self
                l.save()
            for r in rels.filter(model__icontains=e_b):
                lst_ents_rel = str(r).split()
                if lst_ents_rel[0] == lst_ents_rel[1]:
                    q_d = {"related_{}A".format(e_b.lower()): ent}
                    k = r.model_class().objects.filter(**q_d)
                    for t in k:
                        setattr(t, "related_{}A".format(e_a.lower()), self)
                        t.save()
                    q_d = {"related_{}B".format(e_b.lower()): ent}
                    k = r.model_class().objects.filter(**q_d)
                    for t in k:
                        setattr(t, "related_{}B".format(e_a.lower()), self)
                        t.save()
                else:
                    q_d = {"related_{}".format(e_b.lower()): ent}
                    k = r.model_class().objects.filter(**q_d)
                    for t in k:
                        setattr(t, "related_{}".format(e_a.lower()), self)
                        t.save()
            ent.delete()

    def get_serialization(self):
        return EntitySerializer(self).data


@reversion.register()
class Source(models.Model):
    """ Holds information about entities and their relations"""

    orig_filename = models.CharField(max_length=255, blank=True)
    indexed = models.BooleanField(default=False)
    pubinfo = models.CharField(max_length=400, blank=True)
    author = models.CharField(max_length=255, blank=True)
    orig_id = models.PositiveIntegerField(blank=True, null=True)

    def __str__(self):
        if self.author != "" and self.orig_filename != "":
            return "{}, stored by {}".format(self.orig_filename, self.author)
        else:
            return "(ID: {})".format(self.id)


@reversion.register()
class Collection(models.Model):
    """ Allows to group entities and relation. """

    name = models.CharField(max_length=255)
    description = models.TextField(blank=True)
    collection_type = models.ForeignKey(
        CollectionType, blank=True, null=True, on_delete=models.SET_NULL
    )
    groups_allowed = models.ManyToManyField(Group)
    parent_class = models.ForeignKey(
        "self", blank=True, null=True, on_delete=models.CASCADE
    )

    def __str__(self):
        return self.name


@reversion.register()
class Text(models.Model):
    """ Holds unstructured text associeted with
    one ore many entities/relations. """

    kind = models.ForeignKey(TextType, blank=True, null=True, on_delete=models.SET_NULL)
    text = models.TextField(blank=True)
    source = models.ForeignKey(Source, blank=True, null=True, on_delete=models.SET_NULL)

    def __str__(self):
        if self.text != "":
            return "ID: {} - {}".format(self.id, self.text[:25])
        else:
            return "ID: {}".format(self.id)

    def save(self, *args, **kwargs):
        if self.pk is not None:
            orig = Text.objects.get(pk=self.pk)
            if orig.text != self.text and "apis_highlighter" in settings.INSTALLED_APPS:
                ann = Annotation.objects.filter(text_id=self.pk).order_by("start")
                seq = SequenceMatcher(None, orig.text, self.text)
                for a in ann:
                    changed = False
                    count = 0
                    for s in seq.get_matching_blocks():
                        count += 1
                        if s.a <= a.start and (s.a + s.size) >= a.end:
                            a.start += s.b - s.a
                            a.end += s.b - s.a
                            a.save()
                            changed = True
                    if not changed:
                        a.delete()  # TODO: we might want to delete relations as well.
        super().save(*args, **kwargs)


@reversion.register()
class Uri(models.Model):
    uri = models.URLField(blank=True, null=True, unique=True, max_length=255)
    domain = models.CharField(max_length=255, blank=True)
    rdf_link = models.URLField(blank=True)
    entity = models.ForeignKey(
        TempEntityClass, blank=True, null=True, on_delete=models.CASCADE
    )
    # loaded set to True when RDF was loaded and parsed into the data model
    loaded = models.BooleanField(default=False)
    # Timestamp when file was loaded and parsed
    loaded_time = models.DateTimeField(blank=True, null=True)

    def __str__(self):
        return str(self.uri)

    def get_web_object(self):
        result = {
            "relation_pk": self.pk,
            "relation_type": "uri",
            "related_entity": self.entity.name,
            "uri": self.uri,
        }
        return result


@reversion.register()
class UriCandidate(models.Model):
    """Used to store the URI candidates for automatically generated entities.
    """

    uri = models.URLField()
    confidence = models.FloatField(blank=True, null=True)
    responsible = models.CharField(max_length=255)
    entity = models.ForeignKey(
        TempEntityClass, blank=True, null=True, on_delete=models.CASCADE
    )

    @cached_property
    def description(self):
        headers = {"accept": "application/json"}
        cn = TempEntityClass.objects_inheritance.get_subclass(
            id=self.entity_id
        ).__class__.__name__
        for endp in autocomp_settings[cn.title()]:
            url = re.sub(r"/[a-z]+$", "/entity", endp["url"])
            params = {"id": self.uri}
            print(url, params)
            res = requests.get(url, params=params, headers=headers)
            if res.status_code == 200:
                if endp["fields"]["descr"][0] in res.json()["representation"].keys():
                    desc = res.json()["representation"][endp["fields"]["descr"][0]][0][
                        "value"
                    ]
                else:
                    desc = "undefined"
                label = res.json()["representation"][endp["fields"]["name"][0]][0][
                    "value"
                ]
                return (label, desc)


@receiver(post_save, sender=Uri, dispatch_uid="remove_default_uri")
def remove_default_uri(sender, instance, **kwargs):
    if Uri.objects.filter(entity=instance.entity).count() > 1:
        Uri.objects.filter(entity=instance.entity, domain="apis default").delete()



# Generated by Django 2.1.2 on 2018-11-21 15:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('apis_relations', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='placeplace',
            name='related_placeA',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_placeA', to='apis_entities.Place'),
        ),
        migrations.AlterField(
            model_name='placeplace',
            name='related_placeB',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='related_placeB', to='apis_entities.Place'),
        ),
    ]
